@@ -157,11 +157,11 @@ class Base:
 
         items = []
         if rm_id:
-            for doc in self.client[self.db][self.col].find({}):
+            for doc in self.client[self.db][self.col]._find({}):
                 del doc['_id']
                 items.append(doc)
         else:
-            items = list(self.client[self.db][self.col].find({}))
+            items = list(self.client[self.db][self.col]._find({}))
         return items
 
 
@@ -247,7 +247,7 @@ class MI(Base):
         else:
             self.chg_index(index=index)
 
-        d = self.client[self.db][self.col].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
+        d = self.client[self.db][self.col]._find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
         del d['_id']
         return d['date'], d['value']
 
@@ -440,7 +440,7 @@ class C101(Corps):
         else:
             converted_date = date[:4] + '.' + date[4:6] + '.' + date[6:]
         try:
-            d = self.client[self.db][self.col].find({'date': converted_date}).next()
+            d = self.client[self.db][self.col]._find({'date': converted_date}).next()
             del d['_id']
         except StopIteration:
             d = None
@@ -452,7 +452,7 @@ class C101(Corps):
         저장된 모든 데이터를 딕셔너리로 가져와서 리스트로 포장하여 반환한다.
         """
         items = []
-        for doc in self.client[self.db][self.col].find({'date': {'$exists': True}}).sort('date', pymongo.ASCENDING):
+        for doc in self.client[self.db][self.col]._find({'date': {'$exists': True}}).sort('date', pymongo.ASCENDING):
             del doc['_id']
             items.append(doc)
         return items
@@ -483,7 +483,7 @@ class C101(Corps):
             'intro': '한국 및 CE... DP사업으로 구성됨.'}\n
         """
         try:
-            d = self.client[self.db][self.col].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
+            d = self.client[self.db][self.col]._find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
             del d['_id']
         except StopIteration:
             d = {}
@@ -502,7 +502,7 @@ class C108(Corps):
         저장된 모든 데이터를 딕셔너리로 가져와서 리스트로 포장하여 반환한다.
         """
         items = []
-        for doc in self.client[self.db][self.col].find({'날짜': {'$exists': True}}).sort('날짜', pymongo.ASCENDING):
+        for doc in self.client[self.db][self.col]._find({'날짜': {'$exists': True}}).sort('날짜', pymongo.ASCENDING):
             del doc['_id']
             items.append(doc)
         return items
@@ -517,14 +517,14 @@ class C108(Corps):
         """
         # 저장되어 있는 데이터베이스의 최근 날짜를 찾는다.
         try:
-            r_date = self.client[self.db][self.col].find({'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()['날짜']
+            r_date = self.client[self.db][self.col]._find({'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()['날짜']
         except StopIteration:
             # 날짜에 해당하는 데이터가 없는 경우
             return []
 
         # 찾은 날짜를 바탕으로 데이터를 검색하여 리스트로 반환한다.
         r_list = []
-        for r_c108 in self.client[self.db][self.col].find({'날짜': {'$eq': r_date}}):
+        for r_c108 in self.client[self.db][self.col]._find({'날짜': {'$eq': r_date}}):
             del r_c108['_id']
             r_list.append(r_c108)
         return r_list
@@ -779,7 +779,7 @@ class C1034(Corps):
         return target[0], 0 if math.isnan(target[1]) else target[1]
 
     def get_stamp(self) -> datetime.datetime:
-        return self.client[self.db][self.col].find({'stamp': {'$exists': True}}).next()['stamp']
+        return self.client[self.db][self.col]._find({'stamp': {'$exists': True}}).next()['stamp']
 
     def find_cmp(self, title: str) -> float:
         """
@@ -1055,7 +1055,7 @@ class DartByCode(Corps):
 
     def get_data(self) -> pd.DataFrame:
         try:
-            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+            df = pd.DataFrame(list(self.client[self.db][self.col]._find())).drop(columns=['_id'])
         except KeyError:
             df = pd.DataFrame()
         return df
@@ -1099,7 +1099,7 @@ class DartByDate(Base):
         title 인자를 넣으면 타이틀에 해당하는 데이터프레임을 필터링해서 반환한다.
         """
         try:
-            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+            df = pd.DataFrame(list(self.client[self.db][self.col]._find())).drop(columns=['_id'])
         except KeyError:
             df = pd.DataFrame()
         if title != '':
@@ -1137,7 +1137,7 @@ class Noti(Base):
 
     def get_data(self) -> pd.DataFrame:
         try:
-            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+            df = pd.DataFrame(list(self.client[self.db][self.col]._find())).drop(columns=['_id'])
         except KeyError:
             df = pd.DataFrame()
         return df
@@ -1174,7 +1174,7 @@ class EvalByDate(Base):
 
     def get_data(self) -> pd.DataFrame:
         try:
-            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+            df = pd.DataFrame(list(self.client[self.db][self.col]._find())).drop(columns=['_id'])
         except KeyError:
             df = pd.DataFrame()
         return df
