@@ -2,13 +2,16 @@
 
 mongodb atlas 를 manage 하는 클래스 모음
 """
+import math
+from collections import OrderedDict
 import pandas as pd
 import pymongo
 from pymongo import errors
 import datetime
-from typing import Tuple
+from typing import Tuple, List
 from util_hj3415 import utils
-from dart_hj3415 import opendart
+
+from abc import *
 
 import logging
 
@@ -20,6 +23,7 @@ logger.addHandler(ch)
 logger.setLevel(logging.INFO)
 
 ATLAS_ADDR = "mongodb+srv://hj3415:piyrw421@nfs.2mnb4.mongodb.net/005930?retryWrites=true&w=majority"
+INTERNAL_NET_ADDR = 'mongodb://192.168.0.173:27017'
 LOCAL_ADDR = 'mongodb://localhost:27017'
 
 
@@ -29,17 +33,19 @@ def connect_mongo(addr: str, timeout=5):
     # resolve conn error - https://stackoverflow.com/questions/54484890/ssl-handshake-issue-with-pymongo-on-python3
     import certifi
     ca = certifi.where()
-    if addr.startswith('mongodb'):
+    if addr.startswith('mongodb://'):
         # set a some-second connection timeout
+        client = pymongo.MongoClient(addr, serverSelectionTimeoutMS=timeout * 1000)
+    elif addr.startswith('mongodb+srv://'):
         client = pymongo.MongoClient(addr, serverSelectionTimeoutMS=timeout * 1000, tlsCAFile=ca)
-        try:
-            srv_info = client.server_info()
-            conn_str = f"Connect to Mongo Atlas v{srv_info['version']}..."
-            print(conn_str, f"Server Addr : {addr}")
-            return client
-        except Exception:
-            conn_str = f"Unable to connect to the server.(MY IP : {utils.get_ip_addr()})"
-            raise Exception(f"{conn_str} Server Addr : {addr}")
+    try:
+        srv_info = client.server_info()
+        conn_str = f"Connect to Mongo Atlas v{srv_info['version']}..."
+        print(conn_str, f"Server Addr : {addr}")
+        return client
+    except Exception:
+        conn_str = f"Unable to connect to the server.(MY IP : {utils.get_ip_addr()})"
+        raise Exception(f"{conn_str} Server Addr : {addr}")
 
 
 class Atlas:
@@ -172,7 +178,7 @@ class MI(Atlas):
         """
         if self.chk_col():
             d = self.client[self.db][self.index].find({'date': {'$exists': True}}).sort('date',
-                                                                                        pymongo.DESCENDING).next()
+                                                                                         pymongo.DESCENDING).next()
             del d['_id']
             return d['date'], d['value']
 
@@ -516,7 +522,7 @@ class C101(Corps):
         """
         try:
             d = self.client[self.code][self.page].find({'date': {'$exists': True}}).sort('date',
-                                                                                         pymongo.DESCENDING).next()
+                                                                                          pymongo.DESCENDING).next()
             del d['_id']
         except StopIteration:
             d = {}
@@ -692,9 +698,8 @@ class C108(Corps):
         """
         # 저장되어 있는 데이터베이스의 최근 날짜를 찾는다.
         try:
-            r_date = \
-            self.client[self.code][self.page].find({'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()[
-                '날짜']
+            r_date = self.client[self.code][self.page].find(
+                {'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()['날짜']
         except StopIteration:
             # 날짜에 해당하는 데이터가 없는 경우
             return []
@@ -707,14 +712,181 @@ class C108(Corps):
         return r_list
 
 
-class C1034(Corps):
-    pass
+class C1034(Corps, metaclass=ABCMeta):
+    def load(self) -> pd.DataFrame:
+        return super(C1034, self)._load_df()
+
+    def get_all_titles(self) -> list:
+        titles = []
+        for item in self.get_docs():
+            titles.append(item['항목'])
+        return list(set(titles))
+
+    @staticmethod
+    def sum_each_data(data_list: List[dict]) -> dict:
+        """
+        검색된 딕셔너리를 모은 리스트를 인자로 받아서 각각의 기간에 맞춰 합을 구해 하나의 딕셔너리로 반환한다.
+        """
+        sum_dict = {}
+        periods = list(data_list[0].keys())
+        # 여러딕셔너리를 가진 리스트의 합 구하기
+        for period in periods:
+            sum_dict[period] = sum(utils.nan_to_zero(data[period]) for data in data_list)
+        return sum_dict
+
+    @staticmethod
+    def refine_dict(data: dict, refine_words: list) -> dict:
+        """
+        검색된 raw 딕셔너리에서 불필요한 항목들
+        ex - ['_id', '항목', '전년대비', '전년대비1', '전분기대비', '전분기대비1']
+        을 삭제한다.
+        """
+        for del_title in refine_words:
+            try:
+                del data[del_title]
+            except KeyError:
+                pass
+        return data
+
+    def _find(self, title: str, refine_words: list) -> Tuple[List[dict], int]:
+        """
+        title인자에 해당하는 항목을 검색하여 반환한다.
+        c103의 경우는 중복되는 이름의 항목이 있기 때문에
+        이 함수는 반환되는 딕셔너리 리스트와 갯수로 구성되는 튜플을 반환한다.
+        """
+        titles = self.get_all_titles()
+        if title in titles:
+            count = 0
+            data_list = []
+            for data in self.client[self.code][self.page].find({'항목': {'$eq': title}}):
+                count += 1
+                data_list.append(self.refine_dict(data, refine_words))
+            return data_list, count
+        else:
+            raise Exception(f'{title} is not in {titles}')
+
+    @abstractmethod
+    def find(self, title: str) -> dict:
+        pass
+
+    def find_cmp(self, title: str) -> dict:
+        """
+
+        타이틀에 해당하는 전년/분기대비 값을 반환한다.\n
+
+        Args:
+            title (str): 찾고자 하는 타이틀
+
+        Returns:
+            float: 전년/분기대비 증감율
+
+        Note:
+            중복되는 title 은 취급하지 않기로함.\n
+        """
+        data_list, count = self._find(title, ['_id', '항목'])
+        logger.info(data_list)
+        cmp_dict = {}
+        if count == 1:
+            data_dict = data_list[0]
+            for k, v in data_dict.items():
+                if str(k).startswith('전'):
+                    cmp_dict[k] = v
+            return cmp_dict
+        else:
+            raise Exception(f'Not single data..')
+
+    def latest_value(self, title: str, pop_count=2) -> Tuple[str, float]:
+        """가장 최근 년/분기 값
+
+        해당 타이틀의 가장 최근의 년/분기 값을 튜플 형식으로 반환한다.
+
+        Args:
+            title (str): 찾고자 하는 타이틀
+            pop_count: 유효성 확인을 몇번할 것인가
+
+        Returns:
+            tuple: ex - ('2020/09', 39617.5) or ('', 0)
+
+        Note:
+            만약 최근 값이 nan 이면 찾은 값 바로 직전 것을 한번 더 찾아 본다.\n
+            데이터가 없는 경우 ('', nan) 반환한다.\n
+        """
+        def chk_integrity(value) -> bool:
+            if isinstance(value, str):
+                # value : ('Unnamed: 1', '데이터가 없습니다.') 인 경우
+                is_valid = False
+            elif math.isnan(value):
+                # value : float('nan') 인 경우
+                is_valid = False
+            elif value is None:
+                # value : None 인 경우
+                is_valid = False
+            elif value == 0:
+                is_valid = False
+            else:
+                is_valid = True
+            return is_valid
+
+        od = OrderedDict(sorted(self.find(title).items(), reverse=False))
+        logger.debug(f'{title} : {od}')
+
+        for i in range(pop_count):
+            try:
+                d, v = od.popitem(last=True)
+            except KeyError:
+                # when dictionary is empty
+                return '', float('nan')
+            if chk_integrity(v):
+                logger.debug(f'last_one : {v}')
+                return d, v
+
+        return '', float('nan')
+
+    def sum_recent_4q(self, title: str) -> Tuple[str, float]:
+        """최근 4분기 합
+
+        분기 페이지 한정 해당 타이틀의 최근 4분기의 합을 튜플 형식으로 반환한다.
+
+        Args:
+            title (str): 찾고자 하는 타이틀
+
+        Returns:
+            tuple: (계산된 4분기 중 최근분기, 총합)
+
+        Raises:
+            TypeError: 페이지가 q가 아닌 경우 발생
+
+        Note:
+            분기 데이터가 4개 이하인 경우 그냥 최근 연도의 값을 찾아 반환한다.
+        """
+        if self.page.endswith('q'):
+            # 딕셔너리 정렬 - https://kkamikoon.tistory.com/138
+            # reverse = False 이면 오래된것부터 최근순으로 정렬한다.
+            od_q = OrderedDict(sorted(self.find(title).items(), reverse=False))
+            logger.debug(f'{title} : {od_q}')
+
+            if len(od_q) < 4:
+                # od_q의 값이 4개 이하이면 그냥 최근 연도의 값으로 반환한다.
+                y = C1034(client)
+                y.code = self.code
+                y.page = self.col[:-1] + 'y'
+                return y.latest_value(title)
+            else:
+                q_sum = 0
+                latest_period = list(od_q.items())[-1][0]
+                for i in range(4):
+                    # last = True 이면 최근의 값부터 꺼낸다.
+                    d, v = od_q.popitem(last=True)
+                    logger.debug(f'd:{d} v:{v}')
+                    q_sum += 0 if math.isnan(v) else v
+                return str(latest_period), round(q_sum, 2)
+        else:
+            raise TypeError(f'Not support year data..{self.page}')
 
 
 class C103(C1034):
     """C103 컬렉션 관련 클래스
     """
-
     def __init__(self, client, code: str, page: str):
         """
 
@@ -747,6 +919,14 @@ class C103(C1034):
         self.clear_col()
         return super(C103, self)._save_df(c103_data)
 
+    def find(self, title: str) -> dict:
+        """
+        title에 해당하는 항목을 딕셔너리로 반환한다.
+        c103의 경우는 중복된 타이틀을 가지는 항목이 있어 합쳐서 한개의 딕셔너리로 반환한다.
+        """
+        l, c = super(C103, self)._find(title, ['_id', '항목', '전년대비', '전년대비1', '전분기대비', '전분기대비1'])
+        return self.sum_each_data(l)
+
 
 class C104(C1034):
     """C104 컬렉션 관련 클래스
@@ -794,7 +974,9 @@ class C104(C1034):
         return super(C104, self)._save_df(c104_data)
 
     def modify_stamp(self, days_ago: int):
-        # 인위적으로 타임스템프를 수정한다 - 테스트 용도
+        """
+        인위적으로 타임스템프를 수정한다 - 주로 테스트 용도
+        """
         try:
             before = self.client[self.code][self.page].find_one({'항목': 'stamp'})['time']
         except TypeError:
@@ -805,277 +987,20 @@ class C104(C1034):
         after = self.client[self.code][self.page].find_one({'항목': 'stamp'})['time']
         print(f"Stamp changed: {before} -> {after}")
 
-
-# ================================= 준비중 =====================================
-
-
-class C1034_Old(Corps):
-
-    def get_all_title(self) -> list:
+    def get_all_titles(self) -> list:
         """
-
-        중복된 타이틀을 취급하지 않기 위해 타이틀 전체에서 중복된 타이틀을 제거하고 나머지 타이틀 리스트를 반환한다.
-
-        Returns:
-            list: 중복된 항목을 제외한 모든 항목을 담은 리스트
+        상위 C1034클래스에서 c104는 stamp항목이 있기 때문에 삭제하고 리스트로 반환한다.
         """
-        # 항목만 추출한 리스트 만들기
-        title_list = []
-        for item_dict in self.get_all_item():
-            if 'stamp' in item_dict:
-                # stamp 는 넘어가고...
-                pass
-            else:
-                title_list.append(item_dict['항목'])
-
-        # 리스트 중복요소 찾기
-        # https://infinitt.tistory.com/78
-        count = {}
-        for i in title_list:
-            try:
-                count[i] += 1
-            except KeyError:
-                count[i] = 1
-
-        # 중복되지 않는 타이틀만으로 리스트를 만들어 반환하기
-        return_list = []
-        for k, v in count.items():
-            if v == 1:
-                return_list.append(k)
-
-        return return_list
-
-    def get_all(self) -> dict:
-        """
-
-        페이지의 모든 항목에 대한 값을 딕셔너리 형태로 반환.
-
-        Returns:
-            dict: 각 항목을 키로하고 해당 기간별 데이터를 값으로 하는 딕셔너리
-
-        Example:
-            리턴값 예시\n
-            {'*(비지배)당기순이익': {...'2020/12': 1860.0, '2021/12': nan},
-            '*(지배)당기순이익': {...'2020/12': 16021.5,'2021/12': nan},}
-
-        """
-        c1034_list = self.get_all_item()
-        # c104_list 내부에서는 항목, 전년/분기대비, 년/월을 포함하는 딕셔너리가 담겨있다.
-        return_dict = {}
-        not_duplicated_titles = self.get_all_title()
-        for item_dict in c1034_list:
-            if 'stamp' in item_dict:
-                # stamp 는 넘어가고...
-                continue
-            if item_dict['항목'] in not_duplicated_titles:
-                # 중복된 타이틀을 뺀 나머지 항목들은 타이틀을 키로 하고 년/월을 값으로 하는 새로운 딕셔너리로 만든다.
-                temp_dict = {}
-                # 전년/분기대비, 항목타이틀, Unnamed 제거를 위해 새로운 임시 딕셔너리를 만든다.
-                for k, v in item_dict.items():
-                    if k.startswith('전') or k.startswith('항') or k.startswith('Unnamed'):
-                        pass
-                    else:
-                        temp_dict[k] = v
-
-                if item_dict['항목'] in return_dict.keys():
-                    raise KeyError(f"중복된 항목이 있음 : {item_dict['항목']}")
-                else:
-                    return_dict[item_dict['항목']] = temp_dict
-        return return_dict
-
-    def get_all_cmp(self) -> dict:
-        """
-
-        페이지의 모든 항목에 대한 전년/분기대비 값을 딕셔너리 형태로 반환.
-
-        Returns:
-            dict: 각 항목을 키로하고 전분기/년대비 증감 데이터를 값으로 하는 딕셔너리
-
-        Example:
-            리턴값 예시\n
-            {'*(비지배)당기순이익': 26.1,
-            '*(지배)당기순이익': -12.7,
-            '*CAPEX': 8.0}
-
-        """
-        c1034_list = self.get_all_item()
-        # c104_list 내부에서는 항목, 전년/분기대비, 년/월을 포함하는 딕셔너리가 담겨있다.
-        return_dict = {}
-        not_duplicated_titles = self.get_all_title()
-        for item_dict in c1034_list:
-            if 'stamp' in item_dict:
-                # stamp 는 넘어가고...
-                continue
-            if item_dict['항목'] in not_duplicated_titles:
-                temp_dict = {}
-                for k, v in item_dict.items():
-                    if k.startswith('전'):
-                        temp_dict[k] = v
-                # temp_dict 에는 {'전년대비': -1.36, '전년대비1': nan} 또는 {'전분기대비': 14.63} 가 담긴다.
-                if len(temp_dict) == 1:
-                    # {'전분기대비': 14.63}인 경우 ...
-                    return_dict[item_dict['항목']] = temp_dict.popitem()[1]
-                elif len(temp_dict) == 2:
-                    # {'전년대비': -1.36, '전년대비1': nan}인 경우 ...
-                    v = temp_dict.pop('전년대비1')
-                    if math.isnan(v):
-                        return_dict[item_dict['항목']] = temp_dict.pop('전년대비')
-                    else:
-                        return_dict[item_dict['항목']] = v
-                else:
-                    # 데이터가 없는 경우는 nan 으로 세팅한다.
-                    return_dict[item_dict['항목']] = float('nan')
-        return return_dict
-
-    # ========================특정 페이지 관련 함수들=======================
-
-    @staticmethod
-    def _nan_to_zero(target: Tuple[str, float]) -> Tuple[str, float]:
-        """
-
-        실수형 입력값을 받아 nan인 경우 0으로 변환하여 반환하는 유틸함수.
-        """
-        return target[0], 0 if math.isnan(target[1]) else target[1]
-
-    def get_stamp(self) -> datetime.datetime:
-        return self.client[self.db][self.col].find({'stamp': {'$exists': True}}).next()['stamp']
-
-    def find_cmp(self, title: str) -> float:
-        """
-
-        타이틀에 해당하는 전년/분기대비 값을 반환한다.\n
-        해당 타이틀이 없을 경우 - nan 반환함.\n
-
-        Args:
-            title (str): 찾고자 하는 타이틀
-
-        Returns:
-            float: 전년/분기대비 증감율
-
-        Note:
-            중복되는 title 은 취급하지 않기로함.\n
-            get_all_title()함수에서 중복 타이틀 정리함.\n
-        """
-        try:
-            return self.get_all_cmp()[title]
-        except KeyError:
-            return float('nan')
+        titles = super(C104, self).get_all_titles()
+        titles.remove('stamp')
+        return titles
 
     def find(self, title: str) -> dict:
         """
-
-        타이틀에 해당하는 년도, 분기 딕셔너리를 반환한다.\n
-        해당 타이틀이 없을 경우 - {} 반환함.\n
-
-        Args:
-            title (str): 찾고자 하는 타이틀
-
-        Returns:
-            dict: ex - {'2016/12': 51.9',...'2019/12': 4.2, '2020/12': 344.1, '2021/12': nan}
-
-        Note:
-            중복되는 title 은 취급하지 않기로함.\n
-            get_all_title()함수에서 중복 타이틀 정리함.\n
+        title에 해당하는 항목을 딕셔너리로 반환한다.
         """
-        try:
-            return self.get_all()[title]
-        except KeyError:
-            return {}
-
-    def sum_recent_4q(self, title: str, nan_to_zero: bool = False) -> Tuple[str, float]:
-        """최근 4분기 합
-
-        분기 페이지 한정 해당 타이틀의 최근 4분기의 합을 튜플 형식으로 반환한다.
-
-        Args:
-            title (str): 찾고자 하는 타이틀
-            nan_to_zero (bool): 반환값이 nan 인 경우 0으로 바꿀것인지 결정 여부
-
-        Returns:
-            tuple: (계산된 4분기 중 최근분기, 총합)
-
-        Raises:
-            TypeError: 페이지가 q가 아닌 경우 발생
-
-        Note:
-            분기 데이터가 4개 이하인 경우 그냥 최근 연도의 값을 찾아 반환한다.
-        """
-        if self.col.endswith('q'):
-            # 딕셔너리 정렬 - https://kkamikoon.tistory.com/138
-            # reverse = False 이면 오래된것부터 최근순으로 정렬한다.
-            od_q = OrderedDict(sorted(self.find(title=title).items(), reverse=False))
-            logger.info(f'{title} : {od_q}')
-            if len(od_q) < 4:
-                # od_q의 값이 4개 이하이면 그냥 최근 연도의 값으로 반환한다.
-                t = C1034(code=self.db, page=self.col[:-1] + 'y').latest_value(title=title)
-                if nan_to_zero:
-                    return self._nan_to_zero(t)
-                else:
-                    return t
-            else:
-                q_sum = 0
-                last_date = list(od_q.items())[-1][0]
-                for i in range(4):
-                    # last = True 이면 최근의 값부터 꺼낸다.
-                    d, v = od_q.popitem(last=True)
-                    logger.debug(f'd:{d} v:{v}')
-                    q_sum += 0 if math.isnan(v) else v
-                t = str(last_date), round(q_sum, 2)
-                if nan_to_zero:
-                    return self._nan_to_zero(t)
-                else:
-                    return t
-        else:
-            raise TypeError(f'Not support year data..{self.col}')
-
-    def latest_value(self, title: str, nan_to_zero: bool = False) -> Tuple[str, float]:
-        """가장 최근 년/분기 값
-
-        해당 타이틀의 가장 최근의 년/분기 값을 튜플 형식으로 반환한다.
-
-        Args:
-            title (str): 찾고자 하는 타이틀
-            nan_to_zero (bool): 반환값이 nan 인 경우 0으로 바꿀것인지 결정 여부
-
-        Returns:
-            tuple: ex - ('2020/09', 39617.5) or ('', 0)
-
-        Note:
-            만약 최근 값이 nan 이면 찾은 값 바로 직전 것을 한번 더 찾아 본다.\n
-            데이터가 없는 경우 ('', 0) 반환한다.\n
-        """
-        # 딕셔너리 정렬 - https://kkamikoon.tistory.com/138
-        # reverse = False 이면 오래된것부터 최근순으로 정렬한다.
-        od = OrderedDict(sorted(self.find(title=title).items(), reverse=False))
-        logger.info(f'{title} : {od}')
-        try:
-            last_one = od.popitem(last=True)
-        except KeyError:
-            # when dictionary is empty
-            if nan_to_zero:
-                return '', 0
-            else:
-                return '', float('nan')
-        logger.info(f'last_one : {last_one}')
-        if isinstance(last_one[1], str):
-            # last_one : ('Unnamed: 1', '데이터가 없습니다.') 인 경우
-            if nan_to_zero:
-                return '', 0
-            else:
-                return '', float('nan')
-        elif math.isnan(last_one[1]):
-            try:
-                last_one = od.popitem(last=True)
-            except KeyError:
-                # when dictionary is empty
-                if nan_to_zero:
-                    return '', 0
-                else:
-                    return '', float('nan')
-        if nan_to_zero:
-            return self._nan_to_zero(tuple(last_one))
-        else:
-            return tuple(last_one)
+        l, c = super(C104, self)._find(title, ['_id', '항목', '전년대비', '전년대비1', '전분기대비', '전분기대비1'])
+        return l[0]
 
 
 if __name__ == "__main__":
