@@ -207,10 +207,12 @@ class NotiStructure(BaseNoti):
 
 # ===========================================================
 
+sqlite3_path = load_setting().sqlite3_path
+
+
 def chk_deactivate_sqlite3(func):
     def wrapper(*args, **kwargs):
-        path = load_setting()['sqlite3']
-        if path == '' or path is None:
+        if not load_setting().active_sqlite3:
             print('sqlite3 db is deactivated..')
             return None
         else:
@@ -220,7 +222,7 @@ def chk_deactivate_sqlite3(func):
 
 class Base:
     def __init__(self, folder_name: str, db_name: str):
-        self.sqlite3_path = load_setting()['sqlite3']
+        self.sqlite3_path = load_setting().sqlite3_path
 
         if self.sqlite3_path != '':
             self.engine = self.make_engine(folder_name, db_name)
@@ -232,7 +234,7 @@ class Base:
     @chk_deactivate_sqlite3
     def make_engine(folder_name: str, db_name: str):
         # make folder - /db/{folder_name}
-        path = os.path.join(load_setting()['sqlite3'], folder_name)
+        path = os.path.join(sqlite3_path, folder_name)
         if not os.path.isdir(path):
             os.makedirs(path)
         dsn = f"sqlite:///{os.path.join(path, ''.join([db_name, '.db']))}"
